/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.test.junit;

import java.util.Arrays;
import java.util.Locale;

import org.loom.annotation.processor.AbstractPropertyAnnotationProcessor;
import org.loom.annotation.processor.AnnotationProcessor;
import org.loom.annotation.processor.AnnotationProcessorRepository;
import org.loom.annotation.processor.BasicAnnotationProcessor;
import org.loom.annotation.processor.CacheAnnotationProcessor;
import org.loom.annotation.processor.ColumnAnnotationProcessor;
import org.loom.annotation.processor.DateValidationAnnotationProcessor;
import org.loom.annotation.processor.EagerPropertyAnnotationProcessor;
import org.loom.annotation.processor.EventAnnotationProcessor;
import org.loom.annotation.processor.NumberValidationAnnotationProcessor;
import org.loom.annotation.processor.RequiredValidationAnnotationProcessor;
import org.loom.annotation.processor.RestAnnotationProcessor;
import org.loom.annotation.processor.StringValidationAnnotationProcessor;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.config.Config;
import org.loom.i18n.Message;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.i18n.MessagesRepositoryFactory;
import org.loom.i18n.MessagesRepositoryFactoryImpl;
import org.loom.log.Log;
import org.loom.mapping.ActionMappingRepository;
import org.loom.mapping.RestActionMappingFactory;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.util.ClassUtils;
import org.springframework.context.support.StaticApplicationContext;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.mock.web.MockServletContext;


public class TestUtil {
	
	private static MessagesRepositoryFactory defaultMessagesRepositoryFactory;
	
	private static Log log = Log.getLog();
	
	/**
	 * Transform a Messages instance to a plain String using the system default {@link Locale}
	 */
	public static String toString(Messages messages) {
		MessagesRepository repository = createMessagesRepositoryFactory().getRepository(Locale.getDefault());
		StringBuilder builder = new StringBuilder();
		for (Message m : messages.getAllMessages()) {
			builder.append("\n" + m.getPropertyPath() + "=" + repository.translateMessage(m));
		}
		return builder.toString();
	}

	public static MessagesRepositoryFactory createMessagesRepositoryFactory() {
		if (defaultMessagesRepositoryFactory == null) {
			defaultMessagesRepositoryFactory = createMessagesRepositoryFactory("classpath:resources/loom-messages", "classpath:resources/messages");
		}
		return defaultMessagesRepositoryFactory;
	}
	
	private static LoomServletRequestFactory initLoomServletRequestFactory() {
		LoomServletRequestFactory factory = new LoomServletRequestFactory();
		factory.setMessagesRepositoryFactory(createMessagesRepositoryFactory());
		factory.setServletContext(Config.getInstance().getServletContext());
		factory.validate();
		return factory;
	}
	
	public static MessagesRepositoryFactory createMessagesRepositoryFactory(String... locations) {
		MessagesRepositoryFactoryImpl factory = new MessagesRepositoryFactoryImpl();
		factory.setResourceLoader(new PathMatchingResourcePatternResolver());
		// consistent behavior of tests, no matter the O.S. locale
		factory.setDefaultLocale(new Locale("en", "US"));
		factory.setLocations(Arrays.asList(locations));
		factory.afterPropertiesSet();
		return factory;
	}

	/**
	 * Initialize the Config instance if needed
	 */
	public static void initConfig() {
		if (Config.getInstance() == null) {
			Config config = new Config();
			Config.setInstance(config);
			config.setServletContext(new MockServletContext());
			config.setLoomServletRequestFactory(initLoomServletRequestFactory());
			
			PropertyWrapperFactory propertyWrapperFactory = new PropertyWrapperFactory();
			config.setPropertyWrapperFactory(propertyWrapperFactory);
			
			RestActionMappingFactory actionMappingFactory = new RestActionMappingFactory();
			actionMappingFactory.setPropertyWrapperFactory(propertyWrapperFactory);
			actionMappingFactory.setAnnotationProcessorRepository(createAnnotationProcessorRepository());
			actionMappingFactory.afterPropertiesSet();
			
			ActionMappingRepository actionMappingRepository = new ActionMappingRepository();
			actionMappingRepository.setApplicationContext(new StaticApplicationContext());
			actionMappingRepository.setActionMappingFactory(actionMappingFactory);
			actionMappingRepository.afterPropertiesSet();
			
			config.setActionMappingRepository(actionMappingRepository);
			config.validate();
			
		}
	}
	
	// todo: this is a hack. We should find a better way of doing this
	@SuppressWarnings("unchecked")
	public static AnnotationProcessorRepository createAnnotationProcessorRepository() {
		AnnotationProcessorRepository repository = new AnnotationProcessorRepository();
		if (ClassUtils.isPresent("javax.ws.rs.Path")) {
			log.info("Adding JAX-RS annotations support.");
			repository.addAnnotationProcessor(new RestAnnotationProcessor());
		}
		if (ClassUtils.isPresent("javax.persistence.Entity")) {
			log.info("Adding jpa annotations support.");
			repository.addAnnotationProcessor(new ColumnAnnotationProcessor());
			repository.addAnnotationProcessor(new BasicAnnotationProcessor());
		}
		
		// validation processors are specified last, so they override any JPA or hibernate validator
		repository.addAnnotationProcessor(new DateValidationAnnotationProcessor());
		repository.addAnnotationProcessor(new NumberValidationAnnotationProcessor());
		repository.addAnnotationProcessor(new RequiredValidationAnnotationProcessor());
		repository.addAnnotationProcessor(new StringValidationAnnotationProcessor());
		
		// default method annotation processors
		repository.addAnnotationProcessor(new CacheAnnotationProcessor());
		repository.addAnnotationProcessor(new EagerPropertyAnnotationProcessor());
		repository.addAnnotationProcessor(new EventAnnotationProcessor());
		
		// inject propertyWrapperFactory
		PropertyWrapperFactory propertyWrapperFactory = new PropertyWrapperFactory();
		for (AnnotationProcessor processor : repository.getAnnotationProcessors()) {
			if (processor instanceof AbstractPropertyAnnotationProcessor) {
				((AbstractPropertyAnnotationProcessor) processor).setPropertyWrapperFactory(propertyWrapperFactory);
			}
		}
		return repository;
	}
}
